/* ------------------------------------------------------------ */
/*				Include File Definitions						*/
/* ------------------------------------------------------------ */
#include <stdio.h>
#include "platform.h"  //Contains Cache initializing function
#include "xparameters.h"  //The hardware configuration describing constants
#include "xgpio.h"	//GPIO API functions
#include "xintc.h"	//Interrupt Controller API functions
#include "xil_io.h"	//Contains the Xil_Out32 and Xil_In32 functions
#include "mb_interface.h" //Contains the functions for registering the
						  //interrupt controller with the microblaze MP
#include "ac97_demo.h"
#include "ac97.h"
#include <math.h>

/* ------------------------------------------------------------ */
/*				XPAR Constants									*/
/* ------------------------------------------------------------ */
/* All constants used from xparameters.h are located here so 	*/
/* that they may be easily accessed.							*/
/**/

#define AC97_BASEADDR XPAR_DIGILENT_AC97_CNTLR_BASEADDR
#define BTNS_BASEADDR XPAR_PUSH_BUTTONS_5BITS_BASEADDR
#define SW_BASEADDR XPAR_DIP_SWITCHES_8BITS_BASEADDR
#define LED_BASEADDR XPAR_LEDS_8BITS_BASEADDR
#define BTNS_DEVICE_ID XPAR_PUSH_BUTTONS_5BITS_DEVICE_ID
#define INTC_DEVICE_ID XPAR_INTC_0_DEVICE_ID
#define BTNS_IRPT_ID XPAR_INTC_SINGLE_DEVICE_ID
#define DDR2_BASEADDR XPAR_MCB_DDR2_S0_AXI_BASEADDR

/*
 * The pAudioMem constant points to the physical address of the
 * start of the audio data in the DDR2 memory. It is offset from
 * the beginning of DDR to allow room for program memory
 */
#define pAudioData (DDR2_BASEADDR + 0x01000000)

/* ------------------------------------------------------------ */
/*				Global Variables								*/
/* ------------------------------------------------------------ */

volatile u32 lBtnStateOld;
volatile u8 fsRunAction; //flags are set by interrupts to trigger
						 //different actions
#define DOWNSAMPLE 4 //DOWNSAMPLE = 48kHz / sampling rate
#define N_FRAME 10 	//Estimate noise frame

#define GET_DATA_FRAME 8

/* ------------------------------------------------------------ */
/*				Procedure Definitions							*/
/* ------------------------------------------------------------ */

int main()
{
    init_platform();

    //IO
    int Sample_AC97_in  = 0;
    int fft_re;
    int  fft_im;
    int  ifft_re;
    int  ifft_im;
    int Sample_AC97_out = 0;

    int fft_re_in[64];
    int fft_im_in[64];

    //Flag
    int sample_count = 0; //1-32
    int flg_first_64_sample = 0; //first 64 sample pass
    int i;

	for( i = 0; i < 64; i++ ){

	    fft_re_in[i] = 0;
	    fft_im_in[i] = 0;
	}

	/*
	 * Wait for AC97 to become ready
	 */
    while (!(AC97_Link_Is_Ready (AC97_BASEADDR)));

    /*
     * Set TAG to configure codec
     */
    AC97_Set_Tag_And_Id (AC97_BASEADDR, 0xF800);

    /*
     * Enable audio output and set volume
     */
    AC97_Unmute (AC97_BASEADDR, AC97_MASTER_VOLUME_OFFSET);
    AC97_Unmute (AC97_BASEADDR, AC97_HEADPHONE_VOLUME_OFFSET);
    AC97_Unmute (AC97_BASEADDR, AC97_PCM_OUT_VOLUME_OFFSET);

    AC97_Set_Volume (AC97_BASEADDR, AC97_MASTER_VOLUME_OFFSET,
       						     BOTH_CHANNELS, VOLUME_MAX);
    AC97_Set_Volume (AC97_BASEADDR, AC97_HEADPHONE_VOLUME_OFFSET,
       						     BOTH_CHANNELS, VOLUME_MAX);
    AC97_Set_Volume (AC97_BASEADDR, AC97_PCM_OUT_VOLUME_OFFSET,
       						     BOTH_CHANNELS, VOLUME_MAX);


	AC97_Set_Tag_And_Id (AC97_BASEADDR, 0x9800); //Set to Send/Receive data




    while (1)
    {


    	if(Xil_In32(SW_BASEADDR) & bitSw0){

       	AC97_Set_Tag_And_Id (AC97_BASEADDR, 0xF800);

       	AC97_Mute (AC97_BASEADDR, AC97_MIC_VOLUME_OFFSET);
    	//set record gain
    	AC97_Unmute (AC97_BASEADDR, AC97_RECORD_GAIN_OFFSET);
    	AC97_Set_Volume (AC97_BASEADDR, AC97_RECORD_GAIN_OFFSET,BOTH_CHANNELS, 0x04);

       	AC97_Set_Tag_And_Id (AC97_BASEADDR, 0x9800); //Set to Send/Receive data

			Xil_Out32(LED_BASEADDR, bitRecLED ); //turn on LED
			//PLAY Loop
			while(Xil_In32(SW_BASEADDR) & bitSw0){
                    sample_count += 1;


					// Block execution until next frame is ready
                    AC97_Wait_For_New_Frame (AC97_BASEADDR);


					//input sample
					Sample_AC97_in = XIo_In32(AC97_BASEADDR + AC97_PCM_IN_L_OFFSET);
					Sample_AC97_in = Sample_AC97_in >> 2;


					//output win - fft operation Circuit
					XIo_Out32 ((XPAR_WIN_OUT_BASEADDR), Sample_AC97_in);


					/*
					 * usually cycle
					 */
					if(flg_first_64_sample == 64){
						//Swich
						switch (sample_count) {
						case 1:

							//d1
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[0] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[1] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[2] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[3] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[4] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[5] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[6] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[7] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[8] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[9] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[10] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[11] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[12] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[13] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[14] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[15] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[16] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[17] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[18] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[19] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[20] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[21] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[22] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[23] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[24] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[25] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[26] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[27] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[28] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[29] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[30] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[31] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[32] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[33] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[34] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[35] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[36] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[37] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[38] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[39] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[40] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[41] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[42] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[43] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[44] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[45] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[46] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[47] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[48] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[49] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[50] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[51] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[52] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[53] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[54] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[55] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[56] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[57] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[58] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[59] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[60] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[61] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[62] = fft_re;
							 fft_re = XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
							 fft_re_in[63] = fft_re;

							break;

						case 2:
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[0] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[1] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[2] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[3] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[4] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[5] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[6] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[7] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[8] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[9] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[10] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[11] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[12] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[13] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[14] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[15] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[16] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[17] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[18] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[19] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[20] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[21] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[22] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[23] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[24] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[25] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[26] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[27] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[28] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[29] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[30] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[31] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[32] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[33] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[34] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[35] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[36] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[37] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[38] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[39] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[40] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[41] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[42] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[43] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[44] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[45] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[46] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[47] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[48] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[49] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[50] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[51] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[52] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[53] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[54] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[55] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[56] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[57] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[58] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[59] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[60] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[61] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[62] = fft_im;
							 fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
							 fft_im_in[63] = fft_im;

							break;

						case 31:
							ifft_re = fft_im_in[0];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[1];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[2];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[3];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[4];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[5];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[6];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[7];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[8];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[9];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[10];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[11];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[12];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[13];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[14];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[15];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[16];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[17];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[18];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[19];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[20];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[21];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[22];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[23];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[24];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[25];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[26];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[27];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[28];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[29];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[30];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[31];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[32];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[33];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[34];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[35];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[36];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[37];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[38];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[39];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[40];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[41];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[42];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[43];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[44];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[45];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[46];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[47];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[48];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[49];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[50];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[51];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[52];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[53];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[54];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[55];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[56];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[57];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[58];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[59];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[60];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[61];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[62];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							ifft_re = fft_im_in[63];
							XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);

							break;

						case 32:

							ifft_im = fft_re_in[0];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[1];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[2];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[3];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[4];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[5];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[6];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[7];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[8];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[9];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[10];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[11];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[12];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[13];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[14];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[15];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[16];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[17];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[18];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[19];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[20];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[21];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[22];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[23];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[24];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[25];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[26];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[27];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[28];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[29];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[30];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[31];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[32];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[33];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[34];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[35];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[36];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[37];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[38];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[39];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[40];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[41];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[42];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[43];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[44];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[45];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[46];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[47];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[48];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[49];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[50];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[51];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[52];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[53];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[54];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[55];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[56];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[57];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[58];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[59];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[60];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[61];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[62];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							ifft_im = fft_re_in[63];
							XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);

							break;

						default :


							break;
						}//end Swich

						//input from ifft - ho operation Circuit
						Sample_AC97_out = XIo_In32(XPAR_HO_IN_BASEADDR);

						//output sample to AC97
						Sample_AC97_out = Sample_AC97_out << 2;
						XIo_Out32 ((AC97_BASEADDR + AC97_PCM_OUT_L_OFFSET), Sample_AC97_out);
						XIo_Out32 ((AC97_BASEADDR + AC97_PCM_OUT_R_OFFSET), Sample_AC97_out);
					}//end usually cycle

					/*
					 *  first cycle (0 - 63 sample)
					 */
					else{
						// Block execution until next frame is ready
	                    AC97_Wait_For_New_Frame (AC97_BASEADDR);
						AC97_Wait_For_New_Frame (AC97_BASEADDR);
						AC97_Wait_For_New_Frame (AC97_BASEADDR);

	                    flg_first_64_sample++;
					}//end first cycle

					if(sample_count == 32) sample_count = 0;
			}
			//END PLAY Loop
			Xil_Out32(LED_BASEADDR, 0); //turn off LED

			//Set Tag and ID to configure the codec
			AC97_Set_Tag_And_Id (AC97_BASEADDR, 0xF800);
			//Mute
			AC97_Mute (AC97_BASEADDR, AC97_RECORD_GAIN_OFFSET);

			AC97_Set_Tag_And_Id (AC97_BASEADDR, 0x9800); //Set to Send/Receive data
    	}
    }
    cleanup_platform();

    return 0;
}
